<?php

/*
*
* INFOATCLUSTER TECHNOLABS LLP ("COMPANY") CONFIDENTIAL
* Copyright (c) 2016-2017 INFOATCLUSTER TECHNOLABS LLP, All Rights Reserved.
*
* NOTICE:  All information contained herein is, and remains the property of INFOATCLUSTER TECHNOLABS LLP. The intellectual and technical concepts contained
* herein are proprietary to INFOATCLUSTER TECHNOLABS LLP and may be covered by Indian and Foreign Patents, patents in process, and are protected by trade secret or copyright law.
* Dissemination of this information or reproduction of this material is strictly forbidden unless prior written permission is obtained
* from COMPANY.  Access to the source code contained herein is hereby forbidden to anyone except current COMPANY employees, managers or contractors who have executed 
* Confidentiality and Non-disclosure agreements explicitly covering such access.
*
* The copyright notice above does not evidence any actual or intended publication or disclosure  of  this source code, which includes  
* information that is confidential and/or proprietary, and is a trade secret, of  COMPANY.   ANY REPRODUCTION, MODIFICATION, DISTRIBUTION, PUBLIC  PERFORMANCE, 
* OR PUBLIC DISPLAY OF OR THROUGH USE  OF THIS  SOURCE CODE  WITHOUT  THE EXPRESS WRITTEN CONSENT OF COMPANY IS STRICTLY PROHIBITED, AND IN VIOLATION OF APPLICABLE 
* LAWS AND INTERNATIONAL TREATIES.  THE RECEIPT OR POSSESSION OF  THIS SOURCE CODE AND/OR RELATED INFORMATION DOES NOT CONVEY OR IMPLY ANY RIGHTS  
* TO REPRODUCE, DISCLOSE OR DISTRIBUTE ITS CONTENTS, OR TO MANUFACTURE, USE, OR SELL ANYTHING THAT IT  MAY DESCRIBE, IN WHOLE OR IN PART.                
*
*
* @author Mohammed Shayas M K
* 
*/

namespace App\Http\Controllers\Admin;
use Illuminate\Http\Request;
use App\Http\Helper\Helper;
use App\Managers\Schedule\ScheduleManager;
use Illuminate\Support\Facades\Hash;
use App\Managers\User\UserManager;
use Illuminate\Support\Facades\Mail;
use App\Http\Controllers\Controller;
use Illuminate\Support\Facades\Session;
use Illuminate\Support\Facades\Redirect;
use App\Validators\Admin\AdminScheduleValidator;
use App\Managers\Team\TeamManager;
use App\Managers\Player\PlayerManager;
use App\Managers\SchedulePlayers\SchedulePlayersManager;
use App\Managers\Formation\FormationManager;
use App\Managers\SchedulePreMatchFormation\SchedulePreMatchFormationManager;
use App\Managers\PlayerRatings\PlayerRatingsManager;
use App\Managers\PlayerRatingSubstitute\PlayerRatingSubstituteManager;
use App\Managers\Profile\ProfileManager;
use App\Managers\TacticContainer\TacticContainerManager;
use App\Managers\Tactics\TacticsManager;
use Illuminate\Support\Facades\DB;

class AdminScheduleController extends Controller
{
    public function __construct()
    {
        $this->helper = new Helper();
        $this->scheduleValidator = new AdminScheduleValidator();
        $this->scheduleManager = new ScheduleManager();
        $this->teamManager = new TeamManager();
        $this->playerManager = new PlayerManager();
        $this->schedulePlayerManager = new SchedulePlayersManager();
        $this->SchedulePreMatchFormationManager = new SchedulePreMatchFormationManager();
        $this->formation = new FormationManager();
        $this->playerRatings = new PlayerRatingsManager();
        $this->playerRatingSubstitute = new PlayerRatingSubstituteManager();
        $this->profile = new ProfileManager();
        $this->tacticContainerManager = new TacticContainerManager();
        $this->tacticsManager = new TacticsManager();
    }
    public function add()
    {
        $currentTeam = $this->profile->first();
        $teams = $this->teamManager->pluck('name','UUID');
        return view('/admin/schedule/add')->with('teams', $teams)->with('currentTeam', $currentTeam->name);
    }
    
    private function convertYoutube($string) {
        return preg_replace(
            "/\s*[a-zA-Z\/\/:\.]*youtu(be.com\/watch\?v=|.be\/)([a-zA-Z0-9\-_]+)([a-zA-Z0-9\/\*\-\_\?\&\;\%\=\.]*)/i",
            "<iframe src=\"//www.youtube.com/embed/$2\" allowfullscreen></iframe>",
            $string
        );
    }
    
    public function save(Request $request)
    {
        if(!$request->all() || !$request->exists('data'))
        {
            return response()->json(['No Input Found'], 400);
        }
        $data = $request->get('data');
        $data = $this->helper->changeSerialzeArrayValues($data, 'create');
        if(trim($data['youtube']) != '')
        {
            $data['youtube_embed'] = $this->convertYoutube(trim($data['youtube']));
        }

        if(trim($data['highlights']) != '')
        {
            $data['highlights_embed'] = $this->convertYoutube(trim($data['highlights']));
        }

        $validation = $this->scheduleValidator->add($data);
        
        if($validation)
        {
            return response()->json($validation['errors'], 400);
        }
        
        $scheduleID = $this->scheduleManager->add($data);
        if(!$scheduleID)
        {
            return response()->json(['Something went wrong'], 500);
        }
        $schedule = $this->scheduleManager->where('id', $scheduleID)->first();
        
        return response()->json([$schedule->UUID], 200);
    }
    
    public function index()
    {
        $schedules = $this->scheduleManager->with('opposite_team_det')->get();
        return view('/admin/schedule/list')->with('schedules', $schedules);
    }
    
    public function edit($UUID)
    {
        $teams = $this->teamManager->pluck('name','UUID');
        return view('/admin/schedule/edit')->with('UUID', $UUID)->with('teams', $teams);
    }
    
    public function currentSchedule($UUID)
    {
        $schedule = $this->scheduleManager->where('UUID', $UUID)->first()->toArray();
        return response()->json($schedule,200);
    }
    
    public function update(Request $request, $UUID)
    {
        if(!$request->all() || !$request->exists('data'))
        {
            return response()->json(['No Input Found'], 400);
        }
        $data = $request->get('data');
        $data = $this->helper->changeSerialzeArrayValues($data, 'update');
        
        if(trim($data['youtube']) != '')
        {
            $data['youtube_embed'] = $this->convertYoutube(trim($data['youtube']));
        }

        if(trim($data['highlights']) != '')
        {
            $data['highlights_embed'] = $this->convertYoutube(trim($data['highlights']));
        }
        
        $validation = $this->scheduleValidator->update($data, $UUID);
        
        if($validation)
        {
            return response()->json($validation['errors'], 400);
        }
        
        $team = $this->scheduleManager->updateData($data, $UUID);
        if(!$team)
        {
            return response()->json(['Something went wrong'], 500);
        }
        
        return response()->json(['Team Updated Successfully'], 200);
    }
    
    public function addPlayersForSchedule($UUID)
    {
        $formations = $this->formation->get();
        $players = $this->playerManager->with('playerPosition', 'multiple_positions.position')->orderBy('name')->get();
        return view('/admin/schedule/add_players')->with('players', $players)->with('formations', $formations)->with('UUID', $UUID);
    }
    
    public function savePlayersForSchedule(Request $request, $scheduleID)
    {
        if(!$request->all() || !$request->exists('columns'))
        {
            return response()->json(['No Input Found'], 400);
        }
        $columns = $request->get('columns');
        $data = $request->get('data');
        $validation = $this->scheduleValidator->addSchedulePlayers($columns, $data);
        
        if($validation)
        {
            return response()->json($validation['errors'], 400);
        }
        $schedulePlayers = $this->scheduleManager->updateSchedulePlayerData($data, $columns, $scheduleID);
        if(!$schedulePlayers)
        {
            return response()->json(['Something went wrong'], 500);
        }
        
        return response()->json(['Players Updated Successfully'], 200);
    }
    
    public function getPlayersForSchedule($scheduleID)
    {
        $players = $this->schedulePlayerManager->with('player')->where('schedule_id',$scheduleID)->where('deleted_at',  null)->where('inactive', 0)->get();
        return response()->json($players,200);
    }
    
    public function addPreMatchFormationForSchedule($UUID)
    {
        $formations = $this->formation->get();
        $players = $this->playerManager->with('playerPosition')->where('deleted_at',  null)->orderBy('name')->get();
        return view('/admin/schedule/add_formation_pre')->with('players', $players)->with('formations', $formations)->with('UUID', $UUID);
    }
    
    public function savepreMatchFormation(Request $request, $scheduleID)
    {
        if(!$request->exists('formationID') || trim($request->get('formationID')) == '')
        {
            return response()->json(['No Input Found'], 400);
        }
        $formationID = $request->get('formationID');
        $data['schedule_id'] = $scheduleID;
        $data['formation_id'] = $formationID;
        $data['user_id'] = Session::get('admin');
        $data['UUID'] = $this->helper->addUuid();
        $res = $this->SchedulePreMatchFormationManager->add($data);
        if(!$res)
        {
            return response()->json(['Something went wrong'], 500);
        }
        
        return response()->json(['Pre Match Formation Updated Successfully'], 200);
    }
    
    public function getpreMatchFormationInfo($scheduleID)
    {
        $formation = $this->SchedulePreMatchFormationManager->where('schedule_id', $scheduleID)->first();
        if($formation)
        {
            return response()->json($formation,200);
        }
        else {
            return response()->json(array(),400);
        }
    }

    public function getpostMatchFormationInfo($scheduleID)
    {
        $formation = $this->scheduleManager->with('sub1', 'sub2', 'sub3', 'sub4', 'sub5')->where('UUID', $scheduleID)->first();
        if($formation)
        {
            return response()->json($formation,200);
        }
        else {
            return response()->json(array(),400);
        }
    }
    
    public function getRatingsOfPlayersForSchedule($scheduleID)
    {
        //$formation = $this->playerRatings->getPlayerRatingsForSchedule($scheduleID);
        $formation = $this->playerRatings->getPlayerRatingsForSchedule($scheduleID);
        if($formation)
        {
            $res['formation'] = $formation;
            $res['sub'] = $this->playerRatingSubstitute->getSubRatings($scheduleID);

         /*  $scheduleSub1 = $this->scheduleManager->select(DB::raw('SUM(player_ratings_substitute.sub_1)/COUNT(*) as total_sub_rating'), 'schedule.name')->join('player_ratings_substitute', 'schedule.UUID', '=', 'player_ratings_substitute.scheduled_id')->where('schedule.new_player_1', $scheduleID)->groupBy('schedule.UUID')->get();

        $scheduleSub2 = $this->scheduleManager->select(DB::raw('SUM(player_ratings_substitute.sub_2)/COUNT(*) as total_sub_rating'), 'schedule.name')->join('player_ratings_substitute', 'schedule.UUID', '=', 'player_ratings_substitute.scheduled_id')->where('schedule.new_player_2', $scheduleID)->groupBy('schedule.UUID')->get();

        $scheduleSub3 = $this->scheduleManager->select(DB::raw('SUM(player_ratings_substitute.sub_3)/COUNT(*) as total_sub_rating'), 'schedule.name')->join('player_ratings_substitute', 'schedule.UUID', '=', 'player_ratings_substitute.scheduled_id')->where('schedule.new_player_3', $scheduleID)->groupBy('schedule.UUID')->get();

        $scheduleSub4 = $this->scheduleManager->select(DB::raw('SUM(player_ratings_substitute.sub_4)/COUNT(*) as total_sub_rating'), 'schedule.name')->join('player_ratings_substitute', 'schedule.UUID', '=', 'player_ratings_substitute.scheduled_id')->where('schedule.new_player_4', $scheduleID)->groupBy('schedule.UUID')->get();

        $scheduleSub5 = $this->scheduleManager->select(DB::raw('SUM(player_ratings_substitute.sub_5)/COUNT(*) as total_sub_rating'), 'schedule.name')->join('player_ratings_substitute', 'schedule.UUID', '=', 'player_ratings_substitute.scheduled_id')->where('schedule.new_player_5', $scheduleID)->groupBy('schedule.UUID')->get();

       
        $statisticsArray = array();
        $nameArray = array();
        $count = 0;
        $totalRating = 0;

        foreach($statistics AS $each)
        {
            //$res['sub']['rating_count1'] = $each['rating_count'];
            //$totalRating+= $each['rating_count'];
            //$statisticsArray[] = round($each['rating_count'],2);
            $nameArray[] = $each['name'];
            $count++;
        } 

        foreach($scheduleSub1 AS $each)
        {
            $res['sub']['rating_count1'] = $each['rating_count'];
            $totalRating+= $each['total_sub_rating'];
            $statisticsArray[] = round($each['total_sub_rating'],2);
            $nameArray[] = $each['name'];
            $count++;
        }

        foreach($scheduleSub2 AS $each)
        {
            $res['sub']['rating_count2'] = $each['rating_count'];
            $totalRating+= $each['total_sub_rating'];
            $statisticsArray[] = round($each['total_sub_rating'],2);
            $nameArray[] = $each['name'];
            $count++;
        }

        foreach($scheduleSub3 AS $each)
        {
            $res['sub']['rating_count3'] = $each['rating_count'];
            $totalRating+= $each['total_sub_rating'];
            $statisticsArray[] = round($each['total_sub_rating'],2);
            $nameArray[] = $each['name'];
            $count++;
        }

        foreach($scheduleSub4 AS $each)
        {
            $res['sub']['rating_count4'] = $each['rating_count'];
            $totalRating+= $each['total_sub_rating'];
            $statisticsArray[] = round($each['total_sub_rating'],2);
            $nameArray[] = $each['name'];
            $count++;
        }

        foreach($scheduleSub5 AS $each)
        {
            $res['sub']['rating_count5'] = $each['rating_count'];
            $totalRating+= $each['total_sub_rating'];
            $statisticsArray[] = round($each['total_sub_rating'],2);
            $nameArray[] = $each['name'];
            $count++;
        } */
            
            return response()->json($res,200);
        }
        else {
            return response()->json(array(),400);
        }
    }
    
    public function delete($UUID)
    {
        $delete = $this->scheduleManager->deleteData($UUID);
        if(!$delete)
        {
            return response()->json(['Something went wrong'], 500);
        }
        
                    
        return response()->json(['Schedule Deleted Successfully'], 200);
    }
    
    public function getPlayerRating($id)
    {
        
    }
    
    public function deleteSchedulePlayer(Request $request)
    {
        if(!$request->all() || !$request->exists('playerID') || !$request->exists('scheduleID'))
        {
            return response()->json(['No Input Found'], 400);
        }
        
        $playerID = $request->get('playerID');
        $scheduleID = $request->get('scheduleID');
        
        $schedulePlayers = $this->schedulePlayerManager->where('player_id', $playerID)->where('schedule_id', $scheduleID)->first();
        if(!$schedulePlayers)
        {
            return response()->json(['Something Went Wrong. Please reload Page'], 500);
        }
        
        $playerRatings = $this->playerRatings->where('scheduled_players_id', $schedulePlayers['UUID'])->first();
        if($playerRatings)
        {
            return response()->json(['You cannot delete this player, since the player inside this schedule is already voted by some user. Deleting the player can cause severe logical issues inside the database.'], 406);
        }
        
        
        $res = $this->schedulePlayerManager->deletePlayerOfSchedule($playerID, $scheduleID);
        if(!$res)
        {
            return response()->json(['Something Went Wrong'], 500);
        }
        
        return response()->json(['Success'], 200);
    }
    
    public function getAllPlayerPositionRatingOrder($scheduleID)
    {
        $playerPositions = $this->tacticsManager->getTacticsPlayersTotalVotes($scheduleID);
        return response()->json($playerPositions, 200);
    }
}